import pandas as pd
import numpy as np


def gdpdescribe(df, x, y, stats=["mean", "sd", "median"], dec=2):
    """
    Calculates summary statistics for the Numeric Variable x grouping by categorical variable y.

    The function is able to calculate the following descriptive statistics:

      Mean
      Median
      Standard Deviation
      Minimum Value
      Maximum Value
      Range
      Value of 75th percentile
      Value of 25th percentile
      Interquartile range
      Number of Missing values

    Parameters
    ----------
    df: pd.Dataframe
        pandas dataframe with the variables to analyze

    x : str
        column name of a pandas dataframe used to calculate the descriptive statistics

    y: str
      column name of a grouping variable

    dec: int
       number of decimal places to return in the table

    stats: list, default ["mean", "sd", "median"]
         Descriptive statistics to calculate

    Returns
    -------
    pd.Dataframe
        Table with the summary statistics specified as arguments of the function

    Examples
    --------
    >>> gdpdescribe(df, "Value", "Location", stats=["mean", "median", "sd", "min", "max", "range_", "q75", "q25", "iqr", "nas"], dec=3)
    """
    # Avoid scientific notation in the display, use decimal points
    pd.set_option("display.float_format", "{:." f"{dec}" "f}".format)

    # validate inputs arguments of the function

    if isinstance(df, pd.core.frame.DataFrame):
        pass
    else:
        raise ValueError("df should be a pd.DataFrame")

    if x in df.columns.tolist() and y in df.columns.tolist():
        pass
    else:
        raise ValueError("Incorrect Variable names")

    if df[y].nunique() < df.shape[0]:
        pass
    else:
        raise ValueError("Variable Y has unique values in every row, cannot group")

    if df[x].dtype.type == np.float64 or df[x].dtype.type == np.int64:
        pass
    else:
        raise ValueError("Variable X is not numeric")

    if df[y].dtype.type == np.object_:
        pass
    else:
        raise ValueError("Variable Y is not categorical")

    if all(
        x
        in ["mean", "median", "sd", "min", "max", "range_", "q75", "q25", "iqr", "nas"]
        for x in stats
    ):
        pass
    else:
        raise ValueError(
            "The statistic to calculate is not correct! Please enter a valid one"
        )

    # initialize and calculate the stats

    mean = None
    mean = df[[x, y]].groupby([y], dropna=True).mean()
    mean.rename(columns={x: f"mean {x}"}, inplace=True)

    median = None
    median = df[[x, y]].groupby([y], dropna=True).median()
    median.rename(columns={x: f"Median {x}"}, inplace=True)

    sd = None
    sd = df[[x, y]].groupby([y], dropna=True).std()
    sd.rename(columns={x: f"Standard devitation {x}"}, inplace=True)

    min = None
    min = df[[x, y]].groupby([y], dropna=True).min()
    min.rename(columns={x: f"Min {x}"}, inplace=True)

    max = None
    max = df[[x, y]].groupby([y], dropna=True).max()
    max.rename(columns={x: f"Max {x}"}, inplace=True)

    range_ = None
    range_ = df[[x, y]].groupby([y], dropna=True).apply(lambda z: z.max() - z.min())
    range_.rename(columns={x: f"Range {x}"}, inplace=True)

    q75 = None
    q75 = df[[x, y]].groupby([y], dropna=True).quantile(q=0.75)
    q75.rename(columns={x: f"Quartile 75th  {x}"}, inplace=True)

    q25 = None
    q25 = df[[x, y]].groupby([y], dropna=True).quantile(q=0.25)
    q25.rename(columns={x: f"Quartile 25th  {x}"}, inplace=True)

    iqr = (
        df[[x, y]]
        .groupby([y], dropna=True)
        .apply(lambda x: x.quantile(q=0.75) - x.quantile(q=0.25))
    )
    iqr.rename(columns={x: f"IQR  {x}"}, inplace=True)

    nas = None
    nas = df[[x, y]].groupby([y], dropna=True).agg(lambda x: x.isnull().sum())
    nas.rename(columns={x: f"NAs  {x}"}, inplace=True)

    # validate stats to calculate
    out = []

    for i in stats:
        i_1 = eval(i)
        out.append(i_1)

    results = pd.concat(out, axis=1).T
    return results



import pandas as pd
def gdpcleaner(gdpdata: pd.DataFrame):
    """
    Author: Gabe Fairbrother

    Remove spurious columns, Rename relevant columns, Remove NaNs



    Parameters
    ----------
    gdpdata: DataFrame 
    a loaded dataframe based on a downloaded Open Government GDP at basic prices dataset (https://open.canada.ca/en/open-data)


    Returns
    -------

    DataFrame:  A cleaned and simplified DataFrame of the relevant columns for summary and visualization. 
    Possible columns (dataset dependent) include:
        Date: Date of data
        Location: Province or Jurisdiction
        Scale: Scale of the Value column (Percent, Millions, etc)
        Unit: Unit of Measure
        Value: Portion of the GDP for the Location and Date
        NAICS_Class: North American Industry Classification System ID 
        Industry: Industry of Record
        Sub-sector: Non-profit sub-sector
        Special_Industry: Special Industry Aggregate

    Examples
    --------
    >>> result = gdpcleaner(example_data)
    """

    #Check for DataFrame input argument
    if (isinstance(gdpdata, pd.core.frame.DataFrame)):
        pass
    else:
        raise TypeError("Argument must be a Pandas DataFrame")

    cleaned_frame = gdpdata

    #Remove spurious columns
    spurious = ['DGUID', 'UOM_ID', 'SCALAR_ID', 'VECTOR', 'COORDINATE', 
                                          'STATUS', 'SYMBOL', 'TERMINATED', 'DECIMALS', 'Value', 'Seasonal adjustment']
    for column in cleaned_frame.columns :
        if column in spurious:
            cleaned_frame = cleaned_frame.drop(columns=column)


    #Drop any rows with null value
    cleaned_frame = cleaned_frame.dropna()
    
    #Rename relevant columns
    cleaned_frame = cleaned_frame.rename(columns={'REF_DATE': 'Date', 'GEO': 'Location', 
                                            'SCALAR_FACTOR': 'Scale', 'VALUE': 'Value', 'UOM': 'Unit'})

    for column in cleaned_frame.columns:
        if 'NAICS' in column:

            cleaned_frame = cleaned_frame.rename(columns={column: 'NAICS_Class'})
        if 'aggregat' in column: #Not a spelling mistake, there are multiple similar column headers in different datasets
            cleaned_frame = cleaned_frame.rename(columns={column: 'Special_Industry'})

    return cleaned_frame        




import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

def gdpplotter(cleaned_Dataframe):
    """
    Author: Aldo Saltao Barros
    
    Plot a line chart of periods and the selected feature.
    
    Parameters
    ----------
    cleaned_Dataframe : pandas.DataFrame
        This is the name of the dataset that you want to draw a chart. It is an output from the function gdpcleaner
    
    Returns
    -------
        chart of GDP by the Region according to the information within the DataFrame

    Examples
    --------
    >>> gdpplotter(cleaned_df)
    

    """
    # check for DataFrame input argument
    if (type(cleaned_Dataframe) != pd.DataFrame):
         raise TypeError("Argument must be a specific Pandas DataFrame, use the 'gpdcleaner' to create this dataFrame")
    
    # reading the database
    df = cleaned_Dataframe

    #cheking units in Scale
    units_type = df['Scale'].unique()

    # using the same unit for the charts
    if 'millions' in units_type:
        df_filtered = df[df['Scale'] == 'millions']
    elif 'units' in units_type:
        df_filtered = df[df['Scale'] == 'units']
    elif 'units ' in units_type:
        df_filtered = df[df['Scale'] == 'units ']
    else:
        raise TypeError("This dataframe don't have the GDP's units 'millions' or 'units ' or 'units' which are allowed for this analysis. Please check your DataFrame")


    # grouping relevant information
    df_grouped_region = df_filtered.groupby(['Date','Location']).agg(
    Value = ('Value', 'sum')
    ).reset_index()

    # pivoting
    df_grouped_region_pivoted = pd.pivot_table(df_grouped_region, index=['Date'],
                                               values=['Value'], columns=['Location'],fill_value=0, aggfunc=np.sum).reset_index()
    # fixing columns
    df_grouped_region_pivoted.columns = ['_'.join(col).strip() for col in df_grouped_region_pivoted.columns.values]

    # selecting the province names
    provinces = list(df_grouped_region_pivoted.columns)
    year = df['Date'].nunique()

    if "Value_Canada" in provinces and year > 1:
        # Draw a vertical bar chart
        df_grouped_region_pivoted.plot(x='Date_', y='Value_Canada')
        # place legend in center right of plot
        plt.legend(bbox_to_anchor=(1.02, 1), loc='upper left', borderaxespad=0)
        plt.ylabel("GPD (CAD$ - millions)")
        plt.xlabel("Years")
        plt.title("GDP (CAD$ millions) VS Years", loc='center')
        plt.show()

    if "Value_Canada" in provinces and year == 1:
        df_grouped_region_pivoted.plot.bar(x="Date_", y="Value_Canada", rot=70)
        plt.legend(bbox_to_anchor=(1.02, 1), loc='upper left', borderaxespad=0)
        plt.ylabel("GPD (CAD$ - millions)")
        plt.xlabel("Years")
        plt.title(f"GDP (CAD$ millions) VS Year - {year}", loc='center')
        plt.show(block=True)

    if "Value_Canada" not in provinces and year > 1 :
        provinces.remove('Date_')
        df_grouped_region_pivoted.plot(x='Date_', y=provinces)
        #place legend in center right of plot
        plt.legend(bbox_to_anchor=(1.02, 1), loc='upper left', borderaxespad=0)
        plt.ylabel("GPD (CAD$ - millions)")
        plt.xlabel("Years")
        plt.title("GDP (CAD$ millions) VS Years", loc='center')
        plt.show()

    if "Value_Canada" not in provinces and year == 1 :
        provinces.remove('Date_')
        df_grouped_region_pivoted.plot.bar(x="Date_", y=provinces, rot=70)
        plt.legend(bbox_to_anchor=(1.02, 1), loc='upper left', borderaxespad=0)
        plt.ylabel("GPD (CAD$ - millions)")
        plt.xlabel("Years")
        plt.title(f"GDP (CAD$ millions) VS Year - {year}", loc='center')
        plt.show(block=True)



import requests, zipfile
import os
import pandas as pd

def gdpimporter(url, filename=None, filetype='csv'):
    """Download the zipped file, unzip, rename the unzipped files, and
    outputs a dataframe along with the title from meta data.

    This function downloads the zipped data from URL to the local path,
    unzips and renames the files as desired. It then returns the data
    frame along with the title as a tuple.

    Parameters
    ----------
    url : str
        URL to the zip file (ends with .zip)
    filename : str
        the filename that the unzipped csv data (not the MetaData) has. 
        If None, 'open_canada_data.csv' will be the filename. 
        This argument is not useful when filetype is set to 'all'
    filetype : {'csv', 'all'}, default 'csv'
        the types of files that will be extracted. If 'csv', only csv
        files are extracted'. If 'all', files of all types are extracted

    Returns
    -------
    (DataFrame, str) : 
        A tuple containing the dataframe and the title of the data extracted
        from the meta data.


    Examples
    --------
    >>> gdpimporter("https://www150.statcan.gc.ca/n1/tbl/csv/36100400-eng.zip")
    """
    # Exception handling: check if the arguments are feasible
    if (filename != None) and (not isinstance(filename, str)):
        raise TypeError("'filename' should be either None (default) or a string.")
    if filetype not in ['csv', 'all']:
        raise ValueError("'filetype' should either be 'csv' (by default) or 'all'.")
    if (not isinstance(url, str)) or (not url.endswith('.zip')):
        raise ValueError("'url' should be a valid url of a zipfile that ends with '.zip'.")

    zipname = url.split("/")[-1]  ## get the name of original zipfile
    req = requests.get(url)

    with open(zipname, "wb") as code:
        code.write(req.content)
    zipdata = zipfile.ZipFile(zipname)
    zipinfos = zipdata.infolist()
    
    if filetype == "csv":
        # iterate through each file
        for zipinfo in zipinfos:
            # This will do the renaming
            if zipinfo.filename.endswith(".csv") and not zipinfo.filename.endswith("MetaData.csv"):
                if filename == None: 
                    zipinfo.filename = f"open_canada_data.csv"
                else:  ## must be a str
                    zipinfo.filename = f"{filename}.csv"
            zipdata.extract(zipinfo)
    else:
        for zipinfo in zipinfos:
            zipdata.extract(zipinfo)        
    zipdata.close()
    
    for filepath in os.listdir(): 
        if filepath == f"{zipname[:-8]}_MetaData.csv":
            metadata = pd.read_csv(filepath)
            os.remove(filepath) # Clean up the metadata
            continue
        elif filepath.endswith('.zip'):
            os.remove(filepath) # Clean up zip
            continue
        if filename == None:
            if filepath == "open_canada_data.csv":
                data = pd.read_csv(filepath)
        else:
            if filepath == f"{filename}.csv":
                data = pd.read_csv(filepath)
    return data, metadata.index[0]


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['gdphelper']

package_data = \
{'': ['*']}

install_requires = \
['ipykernel>=6.7.0,<7.0.0',
 'matplotlib>=3.5.1,<4.0.0',
 'numpy>=1.22.1,<2.0.0',
 'pandas>=1.3.5,<2.0.0',
 'requests>=2.27.1,<3.0.0']

setup_kwargs = {
    'name': 'gdphelper',
    'version': '1.1.11',
    'description': 'A package for streamlining EDA processes for basic Data Analysis',
    'long_description': '[![Documentation Status](https://readthedocs.org/projects/gdphelper/badge/?version=latest)](https://gdphelper.readthedocs.io/en/latest/?badge=latest)\n[![codecov](https://codecov.io/gh/UBC-MDS/gdphelper/branch/main/graph/badge.svg?token=dZEs5iPrE5)](https://codecov.io/gh/UBC-MDS/gdphelper)  \n\n# gdphelper\n\nThis package is designed to take the url of any of the several dozen GDP-related csv datasets from the [Canadian Government Open Data Portal](https://open.canada.ca/en/open-data) and download, clean load, summarize and visualize the data contained within.  \n\nIt contains 4 functions:\n\n`gdpimporter`: Downloads the zipped data, extracts, renames the appropriate csv, and returns a dataframe along with the title from the meta data.    \n`gdpcleaner`: Loads the data, removes spurious columns, renames used columns, scrubs and data issues. Returns a basic data frame and some category flags.   \n`gdpdescribe` : Evaluates the data category and generates summary statistics by year, region, industry, etc.  \n`gdpplotter`: Generates a set of visualizations of the data set according to the user\'s choices.\n\nThis package is built upon a bunch of popular packages in Python ecosystem, including\n`zipfile`, `matplotlib`, and  `pandas.` What makes this package unique is that it incorporates the common functionalities and streamlines the workflow from downloading the data to performing simple EDA, specifically for the GDP-related data from the Canadian Government Open Data Portal.\n\n## Installation\n\n```bash\n$ pip install gdphelper\n\n```\n\n## Usage\n```python\nfrom gdphelper import gdpimporter\nfrom gdphelper import gdpcleaner\nfrom gdphelper import gdpdescribe\nfrom gdphelper import gdpplotter\n\nURL = "https://www150.statcan.gc.ca/n1/tbl/csv/36100400-eng.zip"\ndata_frame, title = gdpimporter.gdpimporter(URL)\nclean_frame = gdpcleaner.gdpcleaner(data_frame)\ngdpdescribe.gdpdescribe(clean_frame, "Value", "Location", stats=["mean", "median", "sd", "min", "max", "range_"], dec=2)\ngdpplotter.gdpplotter(clean_frame)\n```\n\nfor more detailed documentation, see: https://gdphelper.readthedocs.io/en/latest/\n\n## Contributors\n\n- Aldo Barros          aldosaltao@gmail.com\n- Gabe Fairbrother     gfairbrother@gmail.com\n- Wanying Ye           wanying.ye2020@gmail.com\n- Ramiro Mejia         ramiromejiap@gmail.com\n\n## Contributing\n\nInterested in contributing? Check out the [contributing guidelines](https://github.com/UBC-MDS/Group_03_GOV_CA_GDP_HELPER/blob/main/CONTRIBUTING.md). Please note that this project is released with a [Code of Conduct](https://github.com/UBC-MDS/Group_03_GOV_CA_GDP_HELPER/blob/main/CONDUCT.md). By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`gdphelper` was created by Aldo Barros, Gabriel Fairbrother, Ramiro Mejia, Wanying Ye. It is licensed under the terms of the MIT license.\n\n## Credits\n\n`gdphelper` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n',
    'author': 'Aldo Barros, Gabriel Fairbrother, Ramiro Mejia, Wanying Ye',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
